@@ -1,26 +1,27 @@
-/* $Id: generate_extra_defs.cc 495 2008-01-10 10:11:00Z murrayc $ */
+/* $Id$ */
 
 /* generate_extra_defs.cc
  *
  * Copyright (C) 2001 The Free Software Foundation
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 
 #include "generate_extra_defs.h"
+#include <algorithm>
 
 std::string get_properties(GType gtype)
 {
@@ -66,11 +67,13 @@ std::string get_properties(GType gtype)
     if(pParamSpec)
     {
       //Name and type:
-      std::string strName = g_param_spec_get_name(pParamSpec);
-      std::string strTypeName = G_PARAM_SPEC_TYPE_NAME(pParamSpec);
-      
+      const std::string strName = g_param_spec_get_name(pParamSpec);
+      const std::string strTypeName = G_PARAM_SPEC_TYPE_NAME(pParamSpec);
+
       const gchar* pchBlurb = g_param_spec_get_blurb(pParamSpec);
-      std::string strDocs = (pchBlurb ? pchBlurb : std::string());
+      std::string strDocs = (pchBlurb) ? pchBlurb : "";
+      // Quick hack to get rid of nested double quotes:
+      std::replace(strDocs.begin(), strDocs.end(), '"', '\'');
 
       strResult += "(define-property " + strName + "\n";
       strResult += "  (of-object \"" + strObjectName + "\")\n";
@@ -100,11 +103,16 @@ std::string get_properties(GType gtype)
   return strResult;
 }
 
-std::string get_type_name(GType gtype) //Adds a * if necessary.
+bool gtype_is_a_pointer(GType gtype)
+{
+  return (g_type_is_a(gtype, G_TYPE_OBJECT) || g_type_is_a(gtype, G_TYPE_BOXED));
+}
+
+std::string get_type_name(GType gtype, GTypeIsAPointerFunc is_a_pointer_func) //Adds a * if necessary.
 {
   std::string strTypeName = g_type_name(gtype);
 
-  if( g_type_is_a(gtype, G_TYPE_OBJECT) || g_type_is_a(gtype, G_TYPE_BOXED) )
+  if (is_a_pointer_func && is_a_pointer_func(gtype))
     strTypeName += "*";  //Add * to show that it's a pointer.
   else if( g_type_is_a(gtype, G_TYPE_STRING) )
     strTypeName = "gchar*"; //g_type_name() returns "gchararray".
@@ -112,9 +120,9 @@ std::string get_type_name(GType gtype) //Adds a * if necessary.
   return strTypeName;
 }
 
-std::string get_type_name_parameter(GType gtype)
+std::string get_type_name_parameter(GType gtype, GTypeIsAPointerFunc is_a_pointer_func)
 {
-  std::string strTypeName = get_type_name(gtype);
+  std::string strTypeName = get_type_name(gtype, is_a_pointer_func);
 
   //All signal parameters that are registered as GTK_TYPE_STRING are actually const gchar*.
   if(strTypeName == "gchar*")
@@ -123,21 +131,24 @@ std::string get_type_name_parameter(GType gtype)
   return strTypeName;
 }
 
-std::string get_type_name_signal(GType gtype)
+std::string get_type_name_signal(GType gtype, GTypeIsAPointerFunc is_a_pointer_func)
 {
-  return get_type_name_parameter(gtype); //At the moment, it needs the same stuff.
+  return get_type_name_parameter(gtype, is_a_pointer_func); //At the moment, it needs the same stuff.
 }
 
 
-std::string get_signals(GType gtype)
+std::string get_signals(GType gtype, GTypeIsAPointerFunc is_a_pointer_func)
 {
   std::string strResult;
   std::string strObjectName = g_type_name(gtype);
 
   gpointer gclass_ref = 0;
+  gpointer ginterface_ref = 0;
 
   if(G_TYPE_IS_OBJECT(gtype))
     gclass_ref = g_type_class_ref(gtype); //Ensures that class_init() is called.
+  else if(G_TYPE_IS_INTERFACE(gtype))
+    ginterface_ref = g_type_default_interface_ref(gtype); //install signals.
 
   //Get the list of signals:
   guint iCount = 0;
@@ -162,7 +173,7 @@ std::string get_signals(GType gtype)
       g_signal_query(signal_id, &signalQuery);
 
       //Return type:
-      std::string strReturnTypeName = get_type_name_signal( signalQuery.return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE ); //The type is mangled with a flag. Hacky.
+      std::string strReturnTypeName = get_type_name_signal( signalQuery.return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE, is_a_pointer_func ); //The type is mangled with a flag. Hacky.
       //bool bReturnTypeHasStaticScope = (signalQuery.return_type & G_SIGNAL_TYPE_STATIC_SCOPE) == G_SIGNAL_TYPE_STATIC_SCOPE;
       strResult += "  (return-type \"" + strReturnTypeName + "\")\n";
 
@@ -203,7 +214,7 @@ std::string get_signals(GType gtype)
           pchNum = 0;
 
           //Just like above, for the return type:
-          std::string strTypeName = get_type_name_signal( typeParamMangled & ~G_SIGNAL_TYPE_STATIC_SCOPE ); //The type is mangled with a flag. Hacky.
+          std::string strTypeName = get_type_name_signal( typeParamMangled & ~G_SIGNAL_TYPE_STATIC_SCOPE, is_a_pointer_func ); //The type is mangled with a flag. Hacky.
           //bool bReturnTypeHasStaticScope = (typeParamMangled & G_SIGNAL_TYPE_STATIC_SCOPE) == G_SIGNAL_TYPE_STATIC_SCOPE;
 
           strResult += "    '(\"" + strTypeName + "\" \"" + strParamName + "\")\n";
@@ -220,20 +231,22 @@ std::string get_signals(GType gtype)
 
   if(gclass_ref)
     g_type_class_unref(gclass_ref); //to match the g_type_class_ref() above.
+  else if(ginterface_ref)
+    g_type_default_interface_unref(ginterface_ref); // for interface ref above.
 
   return strResult;
 }
 
 
 
-std::string get_defs(GType gtype)
+std::string get_defs(GType gtype, GTypeIsAPointerFunc is_a_pointer_func)
 {
   std::string strObjectName = g_type_name(gtype);
   std::string strDefs = ";; From " + strObjectName + "\n\n";
 
   if(G_TYPE_IS_OBJECT(gtype) || G_TYPE_IS_INTERFACE(gtype))
   {
-    strDefs += get_signals(gtype);
+    strDefs += get_signals(gtype, is_a_pointer_func);
     strDefs += get_properties(gtype);
   }
 
